const fs = require('fs');
const download = require('download-git-repo');
const handlebars = require('handlebars');
const inquirer = require('inquirer');
const ora = require('ora');
const chalk = require('chalk');
const symbols = require('log-symbols');
const NodeCmd = require('node-cmd');
const execSync = require('child_process').execSync;
const Gitlab = require('../lib/gitlab');
const config = require('./aif-tool-config');

module.exports = function (path, projectName, initGitlab) {
  function isYarnAvailable() {
    try {
      execSync('yarn -v', {stdio: 'ignore'});
      return true;
    } catch (e) {
      return false;
    }
  }

  if (!fs.existsSync(projectName)) {
    const tempInitGitLab = initGitlab.toUpperCase();

    const inquirerArray = [
      {
        name: 'version',
        message: '请输入项目版本号',
      },
      {
        name: 'description',
        message: '请输入项目描述',
      },
      {
        name: 'author',
        message: '请输入作者名称',
      },
    ];

    if (tempInitGitLab === 'Y') {
      /**
       * 如果token 空则提示用户输入用户名,密码
       */
      const token = Gitlab.getToken();

      if (token === '') {
        inquirerArray.unshift({
            name: 'username',
            message: '请输入Gitlab用户名',
          },
          {
            type: 'password',
            name: 'password',
            message: '请输入Gitlab密码',
          });
      }
    }

    /**
     * 命令行交互方式获取一些信息
     */
    inquirer.prompt(inquirerArray).then((answers) => {
      // 开始下载
      const spinner = ora('正在下载模板...');
      spinner.start();

      const useYarn = isYarnAvailable();

      console.log(path);

      download(path, projectName, {clone: true}, (err) => {
        if (err) {
          // 下载失败调用
          spinner.fail();
          console.log(symbols.error, chalk.red(err));
        } else {
          // 下载成功调用
          spinner.succeed();
          const fileName = `${projectName}/package.json`;
          const fileLockName = `${projectName}/package-lock.json`;

          const meta = {
            name: (path.indexOf('webpack-npm') > -1 || path.indexOf('webpack-react-module') > -1) ? (config.ORGANIZATION +
              projectName) : projectName,
            version: answers.version,
            description: answers.description,
            author: answers.author,
          };

          if (fs.existsSync(fileName)) {
            const content = fs.readFileSync(fileName).toString();
            const result = handlebars.compile(content)(meta);
            fs.writeFileSync(fileName, result);
          }

          if (fs.existsSync(fileLockName)) {
            const content = fs.readFileSync(fileLockName).toString();
            const result = handlebars.compile(content)(meta);
            fs.writeFileSync(fileLockName, result);
          }

          console.log(symbols.success, chalk.green('项目初始化完成'));

          /**
           * 要先初始化git，在执行包安装，否则语法检测不生效
           */
          if (tempInitGitLab === 'Y') {
            // gitlab仓库创建
            Gitlab.execSyncInitGitLab(
              {username: answers.username, password: answers.password, projectName, description: answers.description});
          }

          const installCmd = useYarn ? 'yarn install' : 'npm install';

          // 开始下载
          const cmdSpinner = ora('正在使用' + (useYarn ? 'yarn' : 'npm') + '安装依赖...');
          // 显示一个等待动画
          cmdSpinner.start();

          // 执行shell命令
          NodeCmd.get(
            `
            cd ${projectName}
            ${installCmd}
            `,
            function (err, data, stderr) {
              if (!err) {
                // 下载成功调用
                cmdSpinner.succeed();

                console.log(symbols.success, chalk.green('依赖安装完成'));
              } else {
                // 安装失败调用
                cmdSpinner.fail();
                console.log(symbols.error, chalk.red(err));
              }
            },
          );
        }
      });
    });
  } else {
    // 错误提示项目已存在，避免覆盖原有项目
    console.log(symbols.error, chalk.red('项目已存在'));
  }
};
