const co = require('co');
const Promise = require('bluebird');
const NodeCmd = require('node-cmd');
const symbols = require('log-symbols');
const ora = require('ora');
const chalk = require('chalk');
const fs = require('fs');
const config = require('../bin/aif-tool-config');
const cleanToken = require('../bin/cleanToken');

/**
 * 执行shell 登录gitlab
 *
 * @param username 用户名
 * @param password 密码
 */
const getSession = function (userName, password) {
  return new Promise(function (resolve, reject) {
    NodeCmd.get(`curl -d "login=${userName}&password=${password}" ${config.GITLABURL}/session`,
      function (err, data) {
        if (!err) {
          fs.writeFileSync(config.TOKENFILENAME, data);

          resolve(JSON.parse(data));
        } else {
          reject(err);
        }
      },
    );
  });
};

/**
 * 执行shell 创建project
 *
 * @param userInfo 用户信息
 */
const createNewProject = function (userInfo, projectInfo) {
  return new Promise(function (resolve, reject) {
    NodeCmd.get(
      `curl -d "name=${projectInfo.projectName}&namespace_id=${config.TDFEGROUPID}&description=${projectInfo.description}&visibility_level=${config.VISIBILITYLEVEL}" ${config.GITLABURL}/projects?private_token=${userInfo.private_token}`,
      function (err, data) {
        if (!err) {
          resolve(JSON.parse(data));
        } else {
          reject(err);
        }
      },
    );
  });
};

/**
 * 执行shell 创建用户
 * @param userInfo
 * @param project
 */
const addProjectMember = function (userInfo, project) {
  return new Promise(function (resolve, reject) {
    NodeCmd.get(
      `curl -d "id=${project.id}&user_id=${userInfo.id}&access_level=${config.ACCESSLEVEL}" ${config.GITLABURL}/projects/${project.id}/members?private_token=${userInfo.private_token}`,
      function (err, data) {
        if (!err) {
          resolve(JSON.parse(data));
        } else {
          reject(err);
        }
      },
    );
  });
};

const initGitlab = function * (projectInfo) {
  const tokenInfo = getToken();

  const {projectName, description} = projectInfo;

  let userInfo = {};

  if (tokenInfo) {
    userInfo = JSON.parse(tokenInfo);
  } else {
    const {username, password} = projectInfo;

    userInfo = yield getSession(username, password);
  }

  if (userInfo.message) {
    console.log(symbols.success, chalk.red('Token获取失败' + userInfo.message));
    cleanToken();
  } else {
    // 开始下载
    const gitInit = ora('正在初始化仓库...');
    gitInit.start();

    const project = yield createNewProject(userInfo, {projectName, description});

    yield addProjectMember(userInfo, project);

    // 成功调用
    gitInit.succeed();

    console.log(symbols.success, chalk.green('仓库初始化完成...'));

    console.log(symbols.success, chalk.green('仓库信息:' + JSON.stringify(project)));

    // 开始下载
    const cmdSpinner = ora('正在提交Gitlab...');
    // 显示一个等待动画
    cmdSpinner.start();

    const {ssh_url_to_repo} = project;

    // 执行shell命令
    NodeCmd.get(
      `
    cd ${projectName}
    git init
    git add .
    git commit -m 'init'
    git remote add origin ${ssh_url_to_repo}
    git push -u origin master
    `,
      function (err, data, stderr) {
        if (!err) {
          // 下载成功调用
          cmdSpinner.succeed();

          console.log(symbols.success, chalk.green('Gitlab初始化提交完成'));
        } else {
          // 安装失败调用
          cmdSpinner.fail();
          console.log(symbols.error, chalk.red(err));
        }
      },
    );
  }
};

/**
 * 获取本地缓存的用户信息
 *
 * @returns {string}
 */
const getToken = function () {
  let private_token = '';

  if (fs.existsSync(config.TOKENFILENAME)) {
    private_token = fs.readFileSync(config.TOKENFILENAME).toString();
  }

  return private_token;
};

const execSyncInitGitLab = function (projectInfo) {
  co(initGitlab(projectInfo));
};

module.exports = {
  execSyncInitGitLab,
  getToken,
};
